package com.ruoyi.web.controller.file;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.SnowFlake;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.file.domain.FilesProject;
import com.ruoyi.file.domain.FilesStorage;
import com.ruoyi.file.domain.FilesTopic;
import com.ruoyi.file.service.IFilesProjectService;
import com.ruoyi.file.service.IFilesStorageService;
import com.ruoyi.file.service.IFilesTopicService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * 项目管理Controller
 *
 * @author cmc
 * @date 2022-09-05
 */
@RestController
@RequestMapping("/file/project")
public class FilesProjectController extends BaseController {
    @Autowired
    private IFilesProjectService filesProjectService;

    @Autowired
    private IFilesTopicService filesTopicService;

    @Autowired
    private IFilesStorageService filesStorageService;

    @Value("${ruoyi.profile}")
    private String profile;

    /**
     * 查询项目管理列表
     */
    @PreAuthorize("@ss.hasPermi('file:project:list')")
    @GetMapping("/list")
    public TableDataInfo list(FilesProject filesProject) {
        List<FilesProject> list = filesProjectService.selectFilesProjectList(filesProject);
        if (filesProject.getStorageAddress() != null && !filesProject.getStorageAddress().equals("")) {
            FilesStorage filesStorage = new FilesStorage();
            filesStorage.setStorageAddress(filesProject.getStorageAddress());
            if (filesStorageService.selectFilesStorageList(filesStorage).size() > 0)
                filesProject.setStorageId(filesStorageService.selectFilesStorageList(filesStorage).get(0).getId());
        }
        if (filesProject.getTopicName() != null && !filesProject.getTopicName().equals("")) {
            FilesTopic filesTopic = new FilesTopic();
            filesTopic.setTopicName(filesProject.getTopicName());
            if (filesTopicService.selectFilesTopicList(filesTopic).size() > 0)
                filesProject.setTopicId(filesTopicService.selectFilesTopicList(filesTopic).get(0).getId());
        }
        return getDataTable(list);
    }

    /**
     * 导出项目管理列表
     */
    @PreAuthorize("@ss.hasPermi('file:project:export')")
    @Log(title = "项目管理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, FilesProject filesProject) {
        List<FilesProject> list = filesProjectService.selectFilesProjectList(filesProject);
        ExcelUtil<FilesProject> util = new ExcelUtil<FilesProject>(FilesProject.class);
        util.exportExcel(response, list, "项目管理数据");
    }

    /**
     * 导出选中项目管理列表
     */
    @PreAuthorize("@ss.hasPermi('file:project:export')")
    @Log(title = "成果表", businessType = BusinessType.EXPORT)
    @PostMapping("/exportBatch/{ids}")
    public void exportBatch(HttpServletResponse response, FilesProject filesProject, @PathVariable("ids") String[] ids)
    {
        List<FilesProject> list = new ArrayList<>();
        for (String projectId : ids)
            list.add(filesProjectService.selectFilesProjectById(projectId));
        ExcelUtil<FilesProject> util = new ExcelUtil<FilesProject>(FilesProject.class);
        util.exportExcel(response, list, "成果表数据");
    }

    /**
     * 下载项目表模版
     */
    @PostMapping("/importTemplate")
    public void importTemplate(HttpServletResponse response)
    {
        ExcelUtil<FilesProject> util = new ExcelUtil<FilesProject>(FilesProject.class);
        util.importTemplateExcel(response, "项目表模板");
    }

    /**
     * 获取项目管理详细信息
     */
    @PreAuthorize("@ss.hasPermi('file:project:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id) {
        return AjaxResult.success(filesProjectService.selectFilesProjectById(id));
    }

    /**
     * 获取专题对应项目列表
     */
    @PreAuthorize("@ss.hasPermi('file:project:query')")
    @GetMapping("/topicName")
    public AjaxResult getProjectByTopic(String topicName) {
        FilesTopic filesTopic = new FilesTopic();
        filesTopic.setTopicName(topicName);
        String topicId = filesTopicService.selectFilesTopicList(filesTopic).get(0).getId();
        FilesProject filesProject = new FilesProject();
        filesProject.setTopicId(topicId);
        return AjaxResult.success(filesProjectService.selectFilesProjectList(filesProject));
    }

    /**
     * 上传项目excel文件
     */
    @PreAuthorize("@ss.hasPermi('file:project:add')")
    @PostMapping("/uploadSheet")
    public AjaxResult uploadProjectSheet(MultipartFile file) throws Exception {
        if (file.isEmpty()) {
            return AjaxResult.error("文件内容为空！");
        } else {
//            File profilePath = new File(profile);
//            if (!profilePath.exists())//若此目录不存在，则创建之
//                profilePath.mkdirs();
//            File transferFile = new File(profilePath + "\\" + file.getOriginalFilename());
//            file.transferTo(transferFile);
//            InputStream in = new FileInputStream(transferFile);
//            XSSFSheet sheetAt = null;
//            HSSFSheet sheetAt1 = null;
//            int rowNumber = 0;
//            boolean xlsx = Objects.requireNonNull(file.getOriginalFilename()).endsWith("xlsx");
//            if (file.getOriginalFilename() != null && file.getOriginalFilename().endsWith("xlsx")) {
//                sheetAt = new XSSFWorkbook(in).getSheetAt(0);
//                rowNumber = sheetAt.getLastRowNum();
//            } else if (file.getOriginalFilename() != null && file.getOriginalFilename().endsWith("xls")) {
//                sheetAt1 = new HSSFWorkbook(in).getSheetAt(0);
//                rowNumber = sheetAt1.getLastRowNum();
//            }
            ExcelUtil<FilesProject> util = new ExcelUtil<FilesProject>(FilesProject.class);
            List<FilesProject> filesProjectList = util.importExcel(file.getInputStream());
            List<String> IdList = new ArrayList<>();
            for (FilesProject fileProject :  filesProjectService.selectFilesProjectList(new FilesProject())) {
                IdList.add(fileProject.getId());
            }
            for (FilesProject fileProject : filesProjectList)
            {
                if (fileProject.getId() == null || !IdList.contains(fileProject.getId()))
                    fileProject.setId(new SnowFlake().generateId());
                if (IdList.contains(fileProject.getId()))
                    filesProjectService.updateFilesProject(fileProject);
                else
                    filesProjectService.insertFilesProject(fileProject);
            }
//            FilesProject filesProject = new FilesProject();
//            for (int i = 0; i < rowNumber - 1; ) {
//                i++;
//                filesProject.setId(new SnowFlake().generateId());
//                filesProject.setProjectNumber(xlsx ? ExcelUtils.getString(sheetAt.getRow(i).getCell(0)) : ExcelUtils.getString(sheetAt1.getRow(i).getCell(0)));
//                filesProject.setProjectName(xlsx ? ExcelUtils.getString(sheetAt.getRow(i).getCell(1)) : ExcelUtils.getString(sheetAt1.getRow(i).getCell(1)));
//                filesProject.setCurrentProgress(xlsx ? ExcelUtils.getString(sheetAt.getRow(i).getCell(2)) : ExcelUtils.getString(sheetAt1.getRow(i).getCell(2)));
//                filesProject.setProjectSource(xlsx ? ExcelUtils.getString(sheetAt.getRow(i).getCell(3)) : ExcelUtils.getString(sheetAt1.getRow(i).getCell(3)));
//                filesProject.setPartyA(xlsx ? ExcelUtils.getString(sheetAt.getRow(i).getCell(4)) : ExcelUtils.getString(sheetAt1.getRow(i).getCell(4)));
//                filesProject.setContactPerson(xlsx ? ExcelUtils.getString(sheetAt.getRow(i).getCell(5)) : ExcelUtils.getString(sheetAt1.getRow(i).getCell(5)));
//                filesProject.setTelephone(xlsx ? ExcelUtils.getString(sheetAt.getRow(i).getCell(6)) : ExcelUtils.getString(sheetAt1.getRow(i).getCell(6)));
//                filesProject.setContractCode(xlsx ? ExcelUtils.getString(sheetAt.getRow(i).getCell(7)) : ExcelUtils.getString(sheetAt1.getRow(i).getCell(7)));
//                filesProject.setContractNumber(xlsx ? ExcelUtils.getString(sheetAt.getRow(i).getCell(8)) : ExcelUtils.getString(sheetAt1.getRow(i).getCell(8)));
//                filesProject.setProjectType(xlsx ? ExcelUtils.getString(sheetAt.getRow(i).getCell(9)) : ExcelUtils.getString(sheetAt1.getRow(i).getCell(9)));
//                filesProject.setExpectedPrice(xlsx ? ExcelUtils.getString(sheetAt.getRow(i).getCell(10)) : ExcelUtils.getString(sheetAt1.getRow(i).getCell(10)));
//                filesProject.setProjectLevel(xlsx ? ExcelUtils.getString(sheetAt.getRow(i).getCell(11)) : ExcelUtils.getString(sheetAt1.getRow(i).getCell(11)));
//                filesProject.setTechnicalDirector(xlsx ? ExcelUtils.getString(sheetAt.getRow(i).getCell(12)) : ExcelUtils.getString(sheetAt1.getRow(i).getCell(12)));
//                filesProject.setTechinicalDesigner(xlsx ? ExcelUtils.getString(sheetAt.getRow(i).getCell(13)) : ExcelUtils.getString(sheetAt1.getRow(i).getCell(13)));
//                filesProject.setQualityInspector(xlsx ? ExcelUtils.getString(sheetAt.getRow(i).getCell(14)) : ExcelUtils.getString(sheetAt1.getRow(i).getCell(14)));
//                filesProject.setUndertakingSector(xlsx ? ExcelUtils.getString(sheetAt.getRow(i).getCell(15)) : ExcelUtils.getString(sheetAt1.getRow(i).getCell(15)));
//                filesProject.setProjectLeader(xlsx ? ExcelUtils.getString(sheetAt.getRow(i).getCell(16)) : ExcelUtils.getString(sheetAt1.getRow(i).getCell(16)));
//                filesProject.setFund(xlsx ? ExcelUtils.getString(sheetAt.getRow(i).getCell(17)) : ExcelUtils.getString(sheetAt1.getRow(i).getCell(17)));
//                filesProject.setApprovalDate(xlsx ? ExcelUtils.getString(sheetAt.getRow(i).getCell(18)) : ExcelUtils.getString(sheetAt1.getRow(i).getCell(18)));
//                filesProject.setApproachTime(xlsx ? ExcelUtils.getString(sheetAt.getRow(i).getCell(19)) : ExcelUtils.getString(sheetAt1.getRow(i).getCell(19)));
//                filesProject.setWithdrawalTime(xlsx ? ExcelUtils.getString(sheetAt.getRow(i).getCell(20)) : ExcelUtils.getString(sheetAt1.getRow(i).getCell(20)));
//                filesProject.setRequiredCompletionDate(xlsx ? ExcelUtils.getString(sheetAt.getRow(i).getCell(21)) : ExcelUtils.getString(sheetAt1.getRow(i).getCell(21)));
//                filesProject.setSubmissionDate(xlsx ? ExcelUtils.getString(sheetAt.getRow(i).getCell(22)) : ExcelUtils.getString(sheetAt1.getRow(i).getCell(22)));
//                filesProject.setLicensePlateNumber(xlsx ? ExcelUtils.getString(sheetAt.getRow(i).getCell(23)) : ExcelUtils.getString(sheetAt1.getRow(i).getCell(23)));
//                filesProject.setMileage(xlsx ? ExcelUtils.getString(sheetAt.getRow(i).getCell(24)) : ExcelUtils.getString(sheetAt1.getRow(i).getCell(24)));
//                filesProject.setFuelExpense(xlsx ? ExcelUtils.getString(sheetAt.getRow(i).getCell(25)) : ExcelUtils.getString(sheetAt1.getRow(i).getCell(25)));
//                filesProject.setMainEquipment(xlsx ? ExcelUtils.getString(sheetAt.getRow(i).getCell(26)) : ExcelUtils.getString(sheetAt1.getRow(i).getCell(26)));
//                filesProject.setProjectParticipants(xlsx ? ExcelUtils.getString(sheetAt.getRow(i).getCell(27)) : ExcelUtils.getString(sheetAt1.getRow(i).getCell(27)));
//                filesProject.setCurrentProcessing(xlsx ? ExcelUtils.getString(sheetAt.getRow(i).getCell(28)) : ExcelUtils.getString(sheetAt1.getRow(i).getCell(28)));
//                filesProject.setCurrentHandler(xlsx ? ExcelUtils.getString(sheetAt.getRow(i).getCell(29)) : ExcelUtils.getString(sheetAt1.getRow(i).getCell(29)));
//                filesProject.setProjectRegistrant(xlsx ? ExcelUtils.getString(sheetAt.getRow(i).getCell(30)) : ExcelUtils.getString(sheetAt1.getRow(i).getCell(30)));
//                filesProject.setRegistrationDate(xlsx ? ExcelUtils.getString(sheetAt.getRow(i).getCell(31)) : ExcelUtils.getString(sheetAt1.getRow(i).getCell(31)));
//                filesProject.setLeadingSector(xlsx ? ExcelUtils.getString(sheetAt.getRow(i).getCell(32)) : ExcelUtils.getString(sheetAt1.getRow(i).getCell(32)));
//                filesProject.setYear(xlsx ? ExcelUtils.getString(sheetAt.getRow(i).getCell(33)).substring(0,4) : ExcelUtils.getString(sheetAt1.getRow(i).getCell(33)).substring(0,4));
//                filesProjectService.insertFilesProject(filesProject);
//            }
            return AjaxResult.success("上传成功");
        }
    }

    /**
     * 新增项目管理
     */
    @PreAuthorize("@ss.hasPermi('file:project:add')")
    @Log(title = "项目管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(FilesProject filesProject, @RequestParam(value = "kmlfile",required = false) MultipartFile uploadProjectKmlFile, @RequestParam(value = "cpfile",required = false) MultipartFile uploadCpKmlFile) throws IOException {
        ConvertToId(filesProject);
        filesProject.setId(new SnowFlake().generateId());
        if (uploadProjectKmlFile(filesProject, uploadProjectKmlFile) )
            return AjaxResult.error("项目范围kml文件内容为空");
        if (uploadCpKmlFile(filesProject, uploadCpKmlFile))
            return AjaxResult.error("项目控制点kml文件内容为空");
        return toAjax(filesProjectService.insertFilesProject(filesProject));
    }

    private boolean uploadProjectKmlFile(FilesProject filesProject, MultipartFile uploadProjectKmlFile) throws IOException {
        if (uploadProjectKmlFile != null) {
            if (!uploadProjectKmlFile.isEmpty()) {
                File profilePath = new File(profile);
                if (!profilePath.exists())//若此目录不存在，则创建之
                    profilePath.mkdirs();
                File transferFile = new File( profilePath + "\\" + uploadProjectKmlFile.getOriginalFilename());
                if (!transferFile.exists())
                    uploadProjectKmlFile.transferTo(transferFile);
                filesProject.setProjectKml(transferFile.getName());
            }
            else
                return true;
        }
        return false;
    }

    private boolean uploadCpKmlFile(FilesProject filesProject, MultipartFile uploadCpKmlFile) throws IOException {
        if (uploadCpKmlFile != null) {
            if (!uploadCpKmlFile.isEmpty()) {
                File profilePath = new File(profile);
                if (!profilePath.exists())//若此目录不存在，则创建之
                    profilePath.mkdirs();
                File transferFile = new File( profilePath + "\\" + uploadCpKmlFile.getOriginalFilename());
                if (!transferFile.exists())
                    uploadCpKmlFile.transferTo(transferFile);
                filesProject.setProjectCp(transferFile.getName());
            }
            else
                return true;
        }
        return false;
    }

    private void ConvertToId(FilesProject filesProject) {
        FilesStorage storage = new FilesStorage();
        storage.setStorageAddress(filesProject.getStorageAddress());
        if (filesProject.getStorageAddress() != null && !filesProject.getStorageAddress().equals("")) {
            filesProject.setStorageId(new SnowFlake().generateId());
            if (filesStorageService.selectFilesStorageListE(storage).size() > 0)
                filesProject.setStorageId(filesStorageService.selectFilesStorageListE(storage).get(0).getId());
            else {
                FilesStorage filesStorage = new FilesStorage();
                filesStorage.setId(filesProject.getStorageId());
                filesStorage.setStorageAddress(filesProject.getStorageAddress());
                filesStorage.setAdministrator(getLoginUser().getUsername());
                filesStorage.setRemark("");
                filesStorageService.insertFilesStorage(filesStorage);
            }
        }
        FilesTopic topic = new FilesTopic();
        topic.setTopicName(filesProject.getTopicName());
        if (filesProject.getTopicName() != null && !filesProject.getTopicName().equals("")) {
            filesProject.setTopicId(new SnowFlake().generateId());
            if (filesTopicService.selectFilesTopicList(topic).size() > 0)
                filesProject.setTopicId(filesTopicService.selectFilesTopicList(topic).get(0).getId());
            else {
                FilesTopic filesTopic = new FilesTopic();
                filesTopic.setId(filesProject.getTopicId());
                filesTopic.setTopicName(filesProject.getTopicName());
                filesTopicService.insertFilesTopic(filesTopic);
            }
        }
    }

    /**
     * 修改项目管理
     */
    @PreAuthorize("@ss.hasPermi('file:project:edit')")
    @Log(title = "项目管理", businessType = BusinessType.UPDATE)
    @PostMapping("/edit")
    public AjaxResult edit(FilesProject filesProject, @RequestParam(value = "kmlfile",required = false) MultipartFile uploadProjectKmlFile, @RequestParam(value = "cpfile",required = false) MultipartFile uploadCpKmlFile) throws IOException {
        ConvertToId(filesProject);
        if (uploadProjectKmlFile(filesProject, uploadProjectKmlFile) )
            return AjaxResult.error("项目范围kml文件内容为空");
        if (uploadCpKmlFile(filesProject, uploadCpKmlFile))
            return AjaxResult.error("项目控制点kml文件内容为空");
        return toAjax(filesProjectService.updateFilesProject(filesProject));
    }

    /**
     * 删除项目管理
     */
    @PreAuthorize("@ss.hasPermi('file:project:remove')")
    @Log(title = "项目管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable String[] ids) {
        return toAjax(filesProjectService.deleteFilesProjectByIds(ids));
    }
    /**
     * 根据项目编号查询项目信息
     */
    @PreAuthorize("@ss.hasPermi('file:project:query')")
    @GetMapping("/query/{projectNumbers}")
    public AjaxResult getInfoByProjectNumber(@PathVariable String[] projectNumbers){
        return AjaxResult.success(filesProjectService.selectFilesProjectByNumbers(projectNumbers));
    }
}