package com.ruoyi.web.controller.file;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.SnowFlake;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.file.domain.FilesTopic;
import com.ruoyi.file.service.IFilesTopicService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 专题管理Controller
 * 
 * @author cmc
 * @date 2022-09-05
 */
@RestController
@RequestMapping("/file/topic")
public class FilesTopicController extends BaseController
{
    @Autowired
    private IFilesTopicService filesTopicService;

    /**
     * 查询专题管理列表
     */
    @PreAuthorize("@ss.hasPermi('file:topic:list')")
    @GetMapping("/list")
    public TableDataInfo list(FilesTopic filesTopic)
    {
        List<FilesTopic> list = filesTopicService.selectFilesTopicList(filesTopic);
        return getDataTable(list);
    }

    /**
     * 导出专题管理列表
     */
    @PreAuthorize("@ss.hasPermi('file:topic:export')")
    @Log(title = "专题管理", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, FilesTopic filesTopic)
    {
        List<FilesTopic> list = filesTopicService.selectFilesTopicList(filesTopic);
        ExcelUtil<FilesTopic> util = new ExcelUtil<FilesTopic>(FilesTopic.class);
        util.exportExcel(response, list, "专题管理数据");
    }

    /**
     * 获取专题管理详细信息
     */
    @PreAuthorize("@ss.hasPermi('file:topic:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") String id)
    {
        return AjaxResult.success(filesTopicService.selectFilesTopicById(id));
    }

    /**
     * 新增专题管理
     */
    @PreAuthorize("@ss.hasPermi('file:topic:add')")
    @Log(title = "专题管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody FilesTopic filesTopic)
    {
        filesTopic.setId(new SnowFlake().generateId());
        return toAjax(filesTopicService.insertFilesTopic(filesTopic));
    }

    /**
     * 修改专题管理
     */
    @PreAuthorize("@ss.hasPermi('file:topic:edit')")
    @Log(title = "专题管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody FilesTopic filesTopic)
    {
        return toAjax(filesTopicService.updateFilesTopic(filesTopic));
    }

    /**
     * 删除专题管理
     */
    @PreAuthorize("@ss.hasPermi('file:topic:remove')")
    @Log(title = "专题管理", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable String[] ids)
    {
        return toAjax(filesTopicService.deleteFilesTopicByIds(ids));
    }
}
